/*******************************************
Integration 1.0 OpenMP Version

Copyright 2010, Tiago Sommer Damasceno and
  The Shodor Education Foundation, Inc.
  
A C implementation of Area Under a Curve.

To run:
./Integration [Starting_Range] [Ending_Range] [Number of Segments]

See the README included in this directory for
more detailed information.
*******************************************/
#include <stdio.h>
#include <stdlib.h>
#include <string.h>
#include <omp.h>
/*
	f(long double *x)
		- Computes the value of x squared and returns the value.
*/
long double f(long double *x){
	return (*x)*(*x);
}

/*
	main(int argc, char **argv)
		- Contains basic instructions to find the area under a curve.
		- Default settings:
			o X Squared function
			o X ranged from 0 to 1
			o Curve divided into 1000 segments
*/
int main(int argc, char **argv){
	long long count, SEGMENTS = 1000;
	long double x, y, dx, area = 0.0, rangeEnd = 1.0, rangeStart = 0.0;

	//Checks for inputs. The program exits if it has less than 3.
	if(argc <= 3 || argc > 4){ printf("Argument Error - Arguments: %i Expected: 3\n",argc-1);exit(0);}
	
	rangeStart  = atof(argv[1]); // First  Argument: starting range
	rangeEnd	= atof(argv[2]); // Second Argument: ending range
	SEGMENTS 	= atoi(argv[3]); // Third  Argument: Number of segments
	// Checks for valid inputs.
	if(rangeEnd <= rangeStart || SEGMENTS < 1){printf("Argument Error - Invalid Argument\n"); exit(0);} 
	
	// Computes DeltaX: dx
	dx = (rangeEnd-rangeStart)/(long double)SEGMENTS;

	/*
		#pragma omp:
			o "parallel for" - Divides the calculation into equal parts 
		according to the number of threads being used
			o private(x,y) - Creates private copies of x and y 
		to avoid "Race Conditions"
			o reduction() - Creates a private copy of area then once all 
			the threads are finished, OpenMP adds all the private 
			copies to main thread's area variable.
	*/
	#pragma omp parallel for private(x,y) reduction(+:area)
	for(count = 0; count < SEGMENTS; count++){
		x = count*dx;
		y = f(&x);
		area += y*dx;
	}
	
	printf("Area: %Lf\n", area);
	
	return(0);
}